/*
 *			PPP CHAP Module
 *
 *	    Written by Toshiharu OHNO (tony-o@iij.ad.jp)
 *
 *   Copyright (C) 1993, Internet Initiative Japan, Inc. All rights reserverd.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Internet Initiative Japan, Inc.  The name of the
 * IIJ may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *	TODO:
 *		o Imprement behaviour as authenticator.
 *		o Imprement retransmission timer.
 */
#include "fsm.h"
#include "chap.h"
#include "lcpproto.h"
#include "lcp.h"
#include "hdlc.h"
#include "phase.h"
#include "vars.h"

static char *chapcodes[] = {
  "???", "CHALLENGE", "RESPONCE", "SUCCESS", "FAILURE"
};

void
ChapOutput(code, id, ptr, count)
u_int code, id;
u_char *ptr;
int count;
{
  int plen;
  struct fsmheader lh;
  struct mbuf *bp;

  plen =  sizeof(struct fsmheader) + count;
  lh.code = code;
  lh.id = id;
  lh.length = htons(plen);
  bp = mballoc(plen, MB_FSM);
  bcopy(&lh, MBUF_CTOP(bp), sizeof(struct fsmheader));
  if (count)
    bcopy(ptr, MBUF_CTOP(bp) + sizeof(struct fsmheader), count);
#ifdef DEBUG
  DumpBp(bp);
#endif
  LogPrintf(LOG_LCP, "ChapOutput: %s\n", chapcodes[code]);
  HdlcOutput(PRI_NORMAL, PROTO_CHAP, bp);
}

void
SendChapChallenge()
{
  logprintf("SendChapChallenge: not implemented, yet.\n");
}

void
RecvChapTalk(chp, bp)
struct fsmheader *chp;
struct mbuf *bp;
{
  int valsize, len;
  int arglen, keylen;
  int namelen;
  char *cp, *argp, *ap, *name, *digest;
  MD5_CTX context;                            /* context */
  char answer[100];

  len = ntohs(chp->length);
#ifdef DEBUG
  logprintf("length: %d\n", len);
#endif
  arglen = len - sizeof(struct fsmheader);
  namelen = strlen(VarAuthName);
  cp = (char *)MBUF_CTOP(bp);
  valsize = *cp++ & 255;
  argp = malloc(1 + valsize + namelen);
  if (argp) {
    if (valsize != 16) {
      LogPrintf(LOG_PHASE, " Bad value size %d\n", valsize);
      return;
    }
    name = cp + valsize;
    name[arglen - valsize - 1] = 0;
    LogPrintf(LOG_PHASE, " Valsize = %d, Name = %s\n", valsize, name);

    /*
     * XXX: We should determine secret key based on given system name.
     */

    switch (chp->code) {
    case CHAP_CHALLENGE:
      digest = argp;
      *digest++ = 16;		/* value size */
      ap = answer;
      *ap++ = chp->id;
      keylen = strlen(VarAuthKey);
      bcopy(VarAuthKey, ap, keylen);
      ap += keylen;
      bcopy(cp, ap, valsize);
      ap += valsize;
      MD5Init(&context);
      MD5Update(&context, answer, ap - answer);
      MD5Final(digest, &context);
      bcopy(VarAuthName, digest + 16, namelen);
      ap += namelen;
      /* Send answer to the peer */
      ChapOutput(CHAP_RESPONSE, chp->id, argp, namelen + 17);
      break;
    case CHAP_RESPONSE:
      /* XXX: Compare with expected answer */
      break;
    }
  }
}

void
RecvChapResult(chp, bp)
struct fsmheader *chp;
struct mbuf *bp;
{
  int len;
  struct lcpstate *lcp = &LcpInfo;

  len = ntohs(chp->length);
#ifdef DEBUG
  logprintf("length: %d\n", len);
#endif
  if (chp->code == CHAP_SUCCESS) {
    if (lcp->auth_iwait == PROTO_CHAP) {
      lcp->auth_iwait = 0;
      if (lcp->auth_ineed == 0)
	NewPhase(PHASE_NETWORK);
    }
  } else {
    /*
     * Maybe, we shoud close LCP. Of cause, peer may take close action, too.
     */
    ;
  }
}

void
ChapInput(struct mbuf *bp)
{
  int len = plength(bp);
  struct fsmheader *chp;

  if (len >= sizeof(struct fsmheader)) {
    chp = (struct fsmheader *)MBUF_CTOP(bp);
    if (len >= ntohs(chp->length)) {
      if (chp->code < 1 || chp->code > 4)
	chp->code = 0;
      LogPrintf(LOG_LCP, "ChapInput: %s\n", chapcodes[chp->code]);

      bp->offset += sizeof(struct fsmheader);
      bp->cnt -= sizeof(struct fsmheader);

      switch (chp->code) {
      case CHAP_CHALLENGE:
      case CHAP_RESPONSE:
	RecvChapTalk(chp, bp);
	break;
      case CHAP_SUCCESS:
      case CHAP_FAILURE:
	RecvChapResult(chp, bp);
	break;
      }
    }
  }
  pfree(bp);
}
