/*
 *			PPP PAP Module
 *
 *	    Written by Toshiharu OHNO (tony-o@iij.ad.jp)
 *
 *   Copyright (C) 1993, Internet Initiative Japan, Inc. All rights reserverd.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Internet Initiative Japan, Inc.  The name of the
 * IIJ may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *	TODO:
 *		o Imprement behaviour as authenticator.
 *		o Imprement enable/disable control.
 *		o Imprement retransmission timer.
 */
#include "fsm.h"
#include "lcp.h"
#include "pap.h"
#include "vars.h"
#include "hdlc.h"
#include "lcpproto.h"
#include "phase.h"

static char *papcodes[] = {
  "???", "REQUEST", "ACK", "NAK"
};

static int papid;

void
SendPapChallenge()
{
  struct fsmheader lh;
  struct mbuf *bp;
  u_char *cp;
  int namelen, keylen, plen;

  namelen = strlen(VarAuthName);
  keylen = strlen(VarAuthKey);
  plen = namelen + keylen + 2;
#ifdef DEBUG
  logprintf("namelen = %d, keylen = %d\n", namelen, keylen);
#endif
  LogPrintf(LOG_PHASE, "PAP: %s (%s)\n", VarAuthName, VarAuthKey);
  lh.code = PAP_REQUEST;
  lh.id = ++papid;
  lh.length = htons(plen + sizeof(struct fsmheader));
  bp = mballoc(plen + sizeof(struct fsmheader), MB_FSM);
  bcopy(&lh, MBUF_CTOP(bp), sizeof(struct fsmheader));
  cp = MBUF_CTOP(bp) + sizeof(struct fsmheader);
  *cp++ = namelen;
  bcopy(VarAuthName, cp, namelen);
  cp += namelen;
  *cp++ = keylen;
  bcopy(VarAuthKey, cp, keylen);
 
  HdlcOutput(PRI_NORMAL, PROTO_PAP, bp);
}

void
PapInput(struct mbuf *bp)
{
  int len = plength(bp);
  struct fsmheader *php;
  struct lcpstate *lcp = &LcpInfo;

  if (len >= sizeof(struct fsmheader)) {
    php = (struct fsmheader *)MBUF_CTOP(bp);
    if (len >= ntohs(php->length)) {
      if (php->code < 1 || php->code > 3)
	php->code = 0;
      LogPrintf(LOG_PHASE, "PapInput: %s\n", papcodes[php->code]);

      switch (php->code) {
      case PAP_REQUEST:
	/* XXX */
	break;
      case PAP_ACK:
	if (lcp->auth_iwait == PROTO_PAP) {
	  lcp->auth_iwait = 0;
	  if (lcp->auth_ineed == 0)
	    NewPhase(PHASE_NETWORK);
	}
	break;
      case PAP_NAK:
	/* XXX: Should close LCP */
	break;
      }
    }
  }
  pfree(bp);
}
